import yargs from 'yargs';

/**
 * CLI options.
 *
 * @public
 */
export interface ParsedArgs {
  bots: Array<string>;
  channelId: string;
  channelTopic: string;
  commandAdmins: Array<string>;
  commandPrefix: string;
  data: string;
  discordToken: string;
  rconHost: string;
  rconPassword: string;
  rconPort: number;
  sqliteDatabase: string;

  /**
   * Placeholder to allow better exit logic.
   */
  help?: boolean;
}

export const APP_NAME = 'conan-discord';
export const ENV_NAME = 'CONAN_DISCORD';

/**
 * Parse CLI options and environment variables.
 *
 * @public
 */
export async function parseArgs(argv: Array<string>): Promise<ParsedArgs> {
  const parser = yargs(argv).usage(`Usage: ${APP_NAME} [options]`)
    .options({
      bots: {
        default: ['discord'] as Array<string>,
        desc: 'bots to launch',
        string: true,
        type: 'array',
      },
      channelId: {
        default: '',
        desc: 'channel identifier',
        type: 'string',
      },
      channelTopic: {
        default: 'players online: %s',
        desc: 'channel topic template',
        type: 'string',
      },
      commandAdmins: {
        default: [] as Array<string>,
        desc: 'admin command users',
        string: true,
        type: 'array',
      },
      commandPrefix: {
        default: '!',
        desc: 'command prefix marker',
        type: 'string',
      },
      data: {
        default: 'memory',
        desc: 'persistent data layer',
        type: 'string',
      },
      discordToken: {
        demandOption: true,
        desc: 'discord bot token',
        type: 'string',
      },
      rconHost: {
        default: '127.0.0.1',
        desc: 'rcon host',
        type: 'string',
      },
      rconPassword: {
        demandOption: true,
        desc: 'rcon password',
        type: 'string',
      },
      rconPort: {
        default: 25575,
        desc: 'rcon port',
        type: 'number',
      },
      sqliteDatabase: {
        default: './bot.db',
        desc: 'sqlite database file',
        type: 'string',
      },
    })
    .env(ENV_NAME)
    .help()
    .exitProcess(false);

  return parser.parse(argv);
}
