import { mustDefault } from '@apextoaster/js-utils';

import { Message } from '../bot/index.js';
import { DataTable, intValue } from '../data/index.js';
import { authorName, parseBody } from '../utils/command.js';
import { Command, CommandContext } from './index.js';

/**
 * Options for recent players command, row limit.
 */
export interface KarmaArgs {
  amount: number;
}

export const KARMA_LIMITS = {
  min: -10,
  max: +10,
};

/**
 * Respond with a pong message. Provides a basic health check.
 */
export const karma: Command = {
  name: 'karma',
  desc: 'karma counter',
  async run(context: CommandContext, message: Message) {
    const [body, args] = await parseBody<KarmaArgs>(message.content, karma, context.args.commandPrefix, {
      amount: {
        default: 0,
        type: 'number',
      },
    });

    const target = args._[0];
    const targetAuthor = await context.bot.getUser(target);
    const karmaTarget = mustDefault(targetAuthor, message.author);
    context.logger.debug({ body, args, karmaTarget }, 'karma target');

    const argsAmount = args.amount;
    const bodyAmount = countKarma(body);
    const change = Math.max(Math.min(argsAmount + bodyAmount, KARMA_LIMITS.max), KARMA_LIMITS.min);
    context.logger.debug({ change, karmaTarget }, 'changing karma');

    const prev = await context.data.get(DataTable.STATE, karmaTarget, karma.name);
    const prevAmount = intValue(prev, 'amount');
    const nextAmount = prevAmount + change;

    await context.data.set(DataTable.STATE, karmaTarget, karma.name, {
      amount: nextAmount.toFixed(0),
    });

    await message.reply(`${authorName(karmaTarget)} has ${nextAmount} karma`);
  },
};

export function countKarma(body: string): number {
  if (body.includes('++')) {
    return Array.from(body).filter((c) => c === '+').length - 1;
  }

  if (body.includes('--')) {
    return 0 - (Array.from(body).filter((c) => c === '-').length + 1);
  }

  return 0;
}
