import { doesExist } from '@apextoaster/js-utils';

import { ParsedArgs } from '../args.js';
import { Author } from '../bot/index.js';
import { MemoryDataLayer } from './memory.js';
import { SqliteDataLayer } from './sqlite.js';

export enum DataTable {
  STATE = 'state',
  SECRET = 'secret',
}

export interface DataValue {
  [key: string]: string;
}

export interface DataLayer {
  connect(): Promise<void>;

  key(scope: DataTable, author: Author, key: string): Promise<string>;
  get(scope: DataTable, author: Author, key: string): Promise<DataValue | undefined>;
  set(scope: DataTable, author: Author, key: string, value: DataValue): Promise<void>;
}

export function intValue(value: DataValue | undefined, key: string): number {
  if (doesExist(value) && doesExist(value[key])) {
    return parseInt(value[key], 10);
  }

  return 0;
}

export const DATA_LAYERS: {[key: string]: new (args: ParsedArgs) => DataLayer} = {
  memory: MemoryDataLayer,
  sqlite: SqliteDataLayer,
};
