import { ParsedArgs } from '../args.js';
import { Author } from '../bot/index.js';
import { authorName } from '../utils/command.js';
import { DataLayer, DataTable, DataValue } from './index.js';

export class MemoryDataLayer implements DataLayer {
  protected data: Map<string, DataValue>;

  constructor(_args: ParsedArgs) {
    this.data = new Map();
  }

  public async connect() {
    this.data.clear();
  }

  public async key(scope: DataTable, author: Author, key: string) {
    return [
      scope,
      authorName(author),
      key,
    ].join(':');
  }

  public async get(scope: DataTable, author: Author, key: string) {
    const compoundKey = await this.key(scope, author, key);
    return this.data.get(compoundKey);
  }

  public async set(scope: DataTable, author: Author, key: string, value: DataValue) {
    const compoundKey = await this.key(scope, author, key);
    this.data.set(compoundKey, value);
  }
}
