import process from 'node:process';

import { ExitCode, main } from './main.js';

/**
 * A Discord chat bot for Conan Exiles game servers.
 *
 * @packageDocumentation
 */

const ARGS_START = 2; // trim the first few, yargs does not like them

/* c8 ignore start */
main(process.argv.slice(ARGS_START)).then((code) => {
  process.exit(code);
}).catch((err) => {
  // eslint-disable-next-line no-console
  console.error('uncaught error from main', err);
  process.exit(ExitCode.ERROR);
});
/* c8 ignore stop */

export { ParsedArgs, parseArgs } from './args.js';
export { Command, CommandContext } from './command/index.js';
export { discordConnect } from './bot/discord.js';
export { Author, Bot, BotContext, BotCtor, Channel, Message } from './bot/index.js';
export { readlineConnect } from './bot/readline.js';
export { authorName, BodyOptions, matchCommands, commandName, parseBody, removeCommandName, StringsOnly } from './utils/command.js';
export { RconClient, rconConnect, sendWithRetry } from './utils/rcon.js';
export { checkAuthor, ROLE_ADMIN_ONLY, Roles, LOCAL_DISCRIMINATOR } from './utils/roles.js';
