import { Author } from '../bot/index.js';
import { authorName } from './command.js';

/**
 * Roles who should be able to execute a command.
 *
 * @public
 */
export interface Roles {
  /**
   * Admin role. Exclusive with the others: if this is true, authors must be a bot admin.
   */
  admin: boolean;

  /**
   * Local role. Inclusive with the others: if this is true, local authors are allowed.
   */
  local: boolean;

  /**
   * Everyone else. Inclusive with the others: if this is true, authors without any other role are allowed.
   */
  other: boolean;
}

/**
 * Discriminator for local authors.
 *
 * @public
 */
export const LOCAL_DISCRIMINATOR = 'local';

/**
 * Role for admin-only commands.
 *
 * @public
 */
export const ROLE_ADMIN_ONLY: Roles = {
  admin: true,
  local: false,
  other: false,
};

/**
 * Check if an author is allowed to execute a command.
 *
 * @public
 */
export function checkAuthor(admins: Array<string>, roles: Roles, author: Author): boolean {
  if (roles.admin) {
    return admins.includes(authorName(author));
  }

  if (roles.local && author.discriminator === LOCAL_DISCRIMINATOR) {
    return true;
  }

  return roles.other;
}
