enum SortOrder {
  BEFORE = -1,
  AFTER = +1,
};

/**
 * Parse an RCON table into a usable dataset.
 */
export function parseTable(input: string): Array<Array<string>> {
  // split lines
  const lines = input.split('\n');

  // split columns
  const [headers, ...fields] = lines.map(split);

  // get index field
  for (const cells of fields) {
    const match = cells[0].match(/^#(\d+) +(.+)$/);
    if (match) {
      const [_full, index, head] = Array.from(match);
      cells.shift();
      cells.unshift(index, head);
    }
  }

  // sort by index
  const sorted = fields.sort((a, b) => {
    if (a[0] <= b[0]) {
      return SortOrder.BEFORE;
    } else {
      return SortOrder.AFTER;
    }
  });

  // build dataset
  const sortedFields = sorted.map((row) => {
    const [_index, ...rest] = row;
    return rest;
  });

  return [
    headers,
    ...sortedFields,
  ];
}

function trim(cell: string) {
  return cell.trim();
}

function len(cell: string) {
  return cell.length > 0;
}

function split(line: string) {
  return line.split('|').map(trim).filter(len);
}
