/* eslint-disable sonarjs/no-duplicate-string */
import { expect } from 'chai';
import { createInterface } from 'node:readline/promises';
import { NullLogger } from 'noicejs';
import { stub } from 'sinon';

import { parseArgs } from '../../src/args.js';
import { BotContext, Channel } from '../../src/bot/index.js';
import { onLine, readlineConnect } from '../../src/bot/readline.js';
import { Command, CommandContext } from '../../src/command/index.js';
import { MemoryDataLayer } from '../../src/data/memory.js';
import { mockBot, mockChannel, mockRcon, REQUIRED_ARGS } from '../helpers.js';

describe('readline bot', () => {
  it('should close the underlying readline instance', async () => {
    const closeStub = stub();
    const factoryStub = stub().returns({
      close: closeStub,
      on: stub().onCall(0).yields('test'),
      setPrompt: stub(),
    });

    const args = await parseArgs([
      ...REQUIRED_ARGS,
      '--commandPrefix="!"',
    ]);
    const context: BotContext = {
      args,
      commands: [],
      data: new MemoryDataLayer(args),
      logger: NullLogger.global,
      rcon: mockRcon(),
    };

    const readline = await readlineConnect(context, factoryStub);

    readline.destroy();

    expect(closeStub).to.have.callCount(1);
  });

  it('should authenticate commands', async () => {
    const args = await parseArgs([
      ...REQUIRED_ARGS,
      '--commandPrefix="!"',
    ]);
    const channel = mockChannel();
    const command: Command = {
      name: 'test',
      desc: 'test',
      auth: stub().resolves(false),
      run: stub().resolves(),
    };
    const context: CommandContext = {
      args,
      bot: mockBot({}),
      commands: [command],
      data: new MemoryDataLayer(args),
      logger: NullLogger.global,
      rcon: mockRcon(),
    };

    const rl = createInterface(process.stdin, process.stdout);

    await onLine(context, channel, rl, '!test');

    rl.close();

    expect(command.run).to.have.callCount(0);
  });

  it('should read lines and turn them into a message from the local user', async () => {
    const args = await parseArgs([
      ...REQUIRED_ARGS,
      '--commandPrefix="!"',
    ]);
    const channel = mockChannel();
    const command: Command = {
      name: 'test',
      desc: '',
      run: stub().resolves(),
    };
    const context: CommandContext = {
      args,
      bot: mockBot({}),
      commands: [command],
      data: new MemoryDataLayer(args),
      logger: NullLogger.global,
      rcon: mockRcon(),
    };

    const rl = createInterface(process.stdin, process.stdout);

    await onLine(context, channel, rl, '!test');

    rl.close();

    expect(command.run).to.have.callCount(1);
  });

  it('should prompt after errors');
});
