import { expect } from 'chai';
import { NullLogger } from 'noicejs';
import { mock } from 'sinon';

import { parseArgs } from '../../src/args.js';
import { Message } from '../../src/bot/index.js';
import { CommandContext } from '../../src/command/index.js';
import { karma } from '../../src/command/karma.js';
import { MemoryDataLayer } from '../../src/data/memory.js';
import { mockAuthor, mockBot, mockChannel, mockRcon, REQUIRED_ARGS } from '../helpers.js';

describe('karma command', () => {
  it('should store karma', async () => {
    const author = mockAuthor();
    const channel = mockChannel();

    const args = await parseArgs([
      ...REQUIRED_ARGS,
      '--commandPrefix="!"',
    ]);
    const context: CommandContext = {
      args,
      bot: {
        ...mockBot(),
        getChannel: mock().atLeast(1).resolves(channel),
        getUser: mock().atLeast(1).resolves(author),
      },
      commands: [],
      data: new MemoryDataLayer(args),
      logger: NullLogger.global,
      rcon: mockRcon(),
    };

    const replyMock = mock().named('Message.reply').resolves();
    const emptyMessage: Message = {
      author,
      channel,
      content: '',
      reply: replyMock,
    };

    await karma.run(context, emptyMessage);

    // start at 0
    expect(replyMock).to.have.been.calledWith('test#0000 has 0 karma');

    replyMock.resetHistory();
    const increaseMessage: Message = {
      author,
      channel,
      content: '!karma ++++',
      reply: replyMock,
    };

    await karma.run(context, increaseMessage);

    // ++(+++)
    expect(replyMock).to.have.been.calledWith('test#0000 has 3 karma');
  });
});
