import { expect } from 'chai';
import { NullLogger } from 'noicejs';
import { mock } from 'sinon';

import { parseArgs } from '../../../src/args.js';
import { Author, Message } from '../../../src/bot/index.js';
import { rconSQL } from '../../../src/command/admin/rcon-sql.js';
import { CommandContext } from '../../../src/command/index.js';
import { MemoryDataLayer } from '../../../src/data/memory.js';
import { mockChannel, mockRcon, REQUIRED_ARGS } from '../../helpers.js';

describe('rcon SQL command', () => {
  it('should execute the given query', async () => {
    const author: Author = {
      discriminator: '0000',
      username: 'test',
    };
    const args = await parseArgs([
      ...REQUIRED_ARGS,
      '--commandPrefix="!"',
    ]);
    const context: CommandContext = {
      args,
      bot: {
        connect: () => Promise.resolve(true),
        destroy: () => { /* noop */ },
        getChannel: mock().resolves(),
        getUser: mock().atLeast(1).resolves(author),
      },
      commands: [],
      data: new MemoryDataLayer(args),
      logger: NullLogger.global,
      rcon: mockRcon(),
    };

    const replyMock = mock().named('Message.reply').resolves();
    const message: Message = {
      author,
      channel: mockChannel(),
      content: '',
      reply: replyMock,
    };

    await rconSQL.run(context, message);

    expect(replyMock).to.have.callCount(1);
  });
});
