import { expect } from 'chai';
import { parseArgs } from '../../src/args.js';
import { DataTable } from '../../src/data/index.js';
import { SqliteDataLayer } from '../../src/data/sqlite.js';
import { Author } from '../../src/index.js';
import { REQUIRED_ARGS } from '../helpers.js';

describe('sqlite data layer', () => {
  it('should set up the schema', async () => {
    const data = new SqliteDataLayer(await parseArgs([
      ...REQUIRED_ARGS,
      '--sqliteDatabase=":memory:"',
    ]));

    await data.connect();

    // TODO: list tables or something?
  });

  it('should store data', async () => {
    const data = new SqliteDataLayer(await parseArgs([
      ...REQUIRED_ARGS,
      '--sqliteDatabase=":memory:"',
    ]));

    await data.connect();

    const author: Author = {
      discriminator: '',
      username: '',
    };
    const value = {
      value: Math.random().toString(),
    };
    await data.set(DataTable.STATE, author, 'test', value);

    const fetched = await data.get(DataTable.STATE, author, 'test');
    expect(fetched).to.deep.equal(value);
  });
});
