import { stub } from 'sinon';

import { Author, Bot, Channel, Message } from '../src/bot/index.js';
import { RconClient } from '../src/utils/rcon.js';

export const REQUIRED_ARGS = [
  '--discordToken=""',
  '--rconPassword=""',
];

export function mockBot(users: Record<string, Author> = {}): Bot {
  return {
    connect: () => Promise.resolve(true),
    destroy: () => { /* noop */ },
    getChannel: (name: string) => Promise.resolve(mockChannel(name)),
    getUser: (name: string) => Promise.resolve(users[name]),
  };
}

export function mockRcon(commands: Record<string, string> = {}): RconClient {
  const client: RconClient = {
    async connect() {
      return client;
    },
    end() {
      // noop
    },
    async send(command: string) {
      return commands[command];
    },
  };

  return client;
}

export function mockAuthor(username = 'test', discriminator = '0000'): Author {
  return {
    discriminator,
    username,
  };
}

export function mockChannel(name = 'test', edit = stub().resolves()): Channel {
  return {
    edit,
    name,
    server: '',
    topic: '',
    type: 'text',
  };
}

export function mockMessage(content: string, author = mockAuthor(), channel = mockChannel()): Message {
  const message: Message = {
    author,
    channel,
    content,
    reply: () => Promise.resolve(message),
  };

  return message;
}
