import { expect } from 'chai';

import { checkAuthor, LOCAL_DISCRIMINATOR } from '../../src/utils/roles.js';

describe('roles utils', () => {
  describe('check author helper', () => {
    describe('admin role', () => {
      it('should require authors be in the admin list', async () => {
        expect(checkAuthor(['test-user#0000'], {
          admin: true,
          local: false,
          other: false,
        }, {
          discriminator: '0000',
          username: 'test-user',
        })).to.equal(true);
      });
    });

    describe('local role', () => {
      it('should allow authors with the local discriminator', async () => {
        expect(checkAuthor([], {
          admin: false,
          local: true,
          other: false,
        }, {
          discriminator: LOCAL_DISCRIMINATOR,
          username: 'test-user',
        })).to.equal(true);

        expect(checkAuthor([], {
          admin: false,
          local: true,
          other: false,
        }, {
          discriminator: '0000',
          username: 'test-user',
        })).to.equal(false);
      });
    });

    describe('other role', () => {
      it('should allow authors with the other role', async () => {
        expect(checkAuthor([], {
          admin: false,
          local: false,
          other: true,
        }, {
          discriminator: '0000',
          username: 'test-user',
        })).to.equal(true);
      });
    });
  });
});
